package com.huigou.topsun.ep.order.application.impl;

import cn.hutool.core.bean.BeanUtil;
import com.huigou.context.Operator;
import com.huigou.context.OrgUnit;
import com.huigou.context.ThreadLocalUtil;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.ep.order.application.EpSaleOrderApplication;
import com.huigou.topsun.ep.order.domain.EpSaleOrderItem;
import com.huigou.topsun.ep.order.domain.vo.EpSaleOrderItemVo;
import com.huigou.topsun.ep.order.domain.vo.EpSaleOrderVo;
import com.huigou.topsun.ep.order.repository.EpSaleOrderRepository;
import com.huigou.topsun.sap.common.HttpClient;
import com.huigou.topsun.ep.order.application.EpSaleOrderItemApplication;
import com.huigou.topsun.ep.order.domain.EpSaleOrder;
import com.huigou.topsun.ep.order.domain.query.EpSaleOrderQueryRequest;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bmp.operator.OperatorApplication;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.uasp.bpm.ProcessAction;
import com.huigou.uasp.bpm.ProcessStartModel;
import com.huigou.uasp.bpm.engine.application.WorkflowApplication;
import com.huigou.util.ClassHelper;
import com.huigou.util.Constants;
import com.huigou.util.SDO;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.activiti.engine.runtime.ProcessInstance;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * @Auther: xin.lu
 * @Date: 2024/03/12/19:17
 * @Description:
 */
@Service("epSaleOrderApplication")
public class EpSaleOrderApplicationImpl extends FlowBroker implements EpSaleOrderApplication {
    @Autowired
    private EpSaleOrderRepository epSaleOrderRepository;
    @Autowired
    private EpSaleOrderItemApplication epSaleOrderItemApplication;
    @Autowired
    private HttpClient httpClient;
    @Resource
    private WorkflowApplication workflowApplication;
    @Resource
    private OperatorApplication operatorApplication;

    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();
        EpSaleOrder epSaleOrder = getBizEntity(EpSaleOrder.class);
        if (epSaleOrder.isNew()) {
            epSaleOrder.setStatusId(BizBillStatus.APPLYING.getId());
        } else {
            epSaleOrder = (EpSaleOrder) commonDomainService.loadAndFillinProperties(epSaleOrder);
        }
        epSaleOrder = epSaleOrderRepository.save(epSaleOrder);
        List<EpSaleOrderItem> epSaleOrderItems = getBizEntities(EpSaleOrderItem.class, "epSaleOrderItems");
        epSaleOrderItemApplication.saveEpSaleOrderItems(epSaleOrder.getId(),epSaleOrderItems);
        return epSaleOrder.getId();
    }
    
    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        // 返回业务数据给流程实例，
        return ClassHelper.toMap(epSaleOrderRepository.getOne(bizId));
    }

    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        BizBillStatus status = approvePassed() ? BizBillStatus.COMPLETED : BizBillStatus.ABORTED;
        EpSaleOrder epSaleOrder = epSaleOrderRepository.findOne(bizId);
        epSaleOrder.setStatusId(status.getId());
        epSaleOrderRepository.save(epSaleOrder);
        /*try {
            String url = SystemCache.getParameter("sap.purchase.url", String.class);
            String execute = httpClient.execute(epSaleOrder, url);
            List<SapResult> resultList = JSONObject.parseArray(execute, SapResult.class);
            SapResult sapResult = resultList.get(0);
            if (!"S".equals(sapResult.getTYPE())){
                throw new RuntimeException("数据传输失败，"+ sapResult.getMESSAGE());
            }
            epSaleOrderRepository.save(epSaleOrder);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }*/
    }

    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        EpSaleOrder epSaleOrder = epSaleOrderRepository.findOne(bizId);
        epSaleOrder.setStatusId(BizBillStatus.ABORTED.getId());
        epSaleOrderRepository.save(epSaleOrder);
    }

    /**
     * @param delegateTask
     * @param destActivityId 回退到的目标节点id
     */
    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        super.onBack(delegateTask, destActivityId);
        if ("Apply".equalsIgnoreCase(destActivityId)) {
            String bizId = delegateTask.getExecution().getProcessBusinessKey();
            EpSaleOrder epSaleOrder = epSaleOrderRepository.findOne(bizId);
            epSaleOrder.setStatusId(BizBillStatus.APPLYING.getId());
            epSaleOrderRepository.save(epSaleOrder);
        }
    }
    
    
    @Override
    public EpSaleOrder findEpSaleOrderById(String id) {
        return epSaleOrderRepository.findOne(id);
    }

    @Override
    public void deleteEpSaleOrderById(List<String> ids) {
        ids.forEach(id->{
            epSaleOrderRepository.delete(id);
        });
    }

    @Override
    public void updateEpSaleOrder(EpSaleOrder epSaleOrder) {
        epSaleOrderRepository.save(epSaleOrder);
    }

    @Override
    public EpSaleOrder saveEpSaleOrder(EpSaleOrder epSaleOrder) {
        return epSaleOrderRepository.save(epSaleOrder);
    }

    @Override
    public Map<String, Object> slicedEpSaleOrderList(EpSaleOrderQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "epSaleOrder");
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        Map<String, Object> map = this.sqlExecutorDao.executeSlicedQuery(queryModel);
        return map;
    }

    @Override
    public void sendEpSaleOrderData(EpSaleOrder costCenter) {

    }

    @Override
    public void saveEpSaleOrderVo(EpSaleOrderVo epSaleOrderVo) {
        EpSaleOrder epSaleOrder = new EpSaleOrder();
        BeanUtil.copyProperties(epSaleOrderVo,epSaleOrder);

        //初始化用户信息
        String createdById = "5EEB2DC861694A65933CB4CE9A8D870A@C18E23AAAA6940E09DF7C122332C51E0";
        Operator operator = operatorApplication.createOperatorByPersonMemberId(createdById);
        epSaleOrder.setDefaultValues(new OrgUnit(operator.getFullId(), operator.getFullName()));

        Map<String, Object> formData = BeanUtil.beanToMap(epSaleOrder);
        String bizId = this.startProcessInstance(EpSaleOrderApplication.PROCESS_DEFINITION_KEY, formData);
        epSaleOrder.setId(bizId);

        epSaleOrder = epSaleOrderRepository.save(epSaleOrder);

        List<EpSaleOrderItemVo> epSaleOrderItemVos = epSaleOrderVo.getEpSaleOrderItemVos();
        List<EpSaleOrderItem> epSaleOrderItems = new ArrayList<>();
        epSaleOrderItemVos.forEach(epSaleOrderItemVo -> {
            EpSaleOrderItem epSaleOrderItem = new EpSaleOrderItem();
            BeanUtil.copyProperties(epSaleOrderItemVo,epSaleOrderItem);
            epSaleOrderItems.add(epSaleOrderItem);
        });
        this.epSaleOrderItemApplication.saveEpSaleOrderItems(epSaleOrder.getId(),epSaleOrderItems);
    }

    public String startProcessInstance(String processDefinitionKey,Map<String, Object> formData){
        Map<String, Object> variables = new HashMap();
        SDO localSdo = this.buildSDO(formData);
        ThreadLocalUtil.putVariable(Constants.SDO, localSdo);
        variables.put("startModel", ProcessStartModel.MANUAL.getId());
        Operator operator = localSdo.getOperator();
        variables.put("executorFullId", operator.getFullId());
        variables.put("executorFullName", operator.getFullName());
        ThreadLocalUtil.putOperator(operator);
        ThreadLocalUtil.putVariable("client.ip", "127.0.0.1");
        ProcessInstance processInstance = this.workflowApplication.startProcessInstanceByKey(processDefinitionKey, variables);
        String businessKey = processInstance.getBusinessKey();
        return businessKey;
    }

    private SDO buildSDO(Map<String, Object> formData) {
        //String createdById = (String) formData.get("createdById");
        //暂定管理员id
        String createdById = "5EEB2DC861694A65933CB4CE9A8D870A@C18E23AAAA6940E09DF7C122332C51E0";
        Operator operator = operatorApplication.createOperatorByPersonMemberId(createdById);
        SDO localSdo = new SDO();
        // 将业务数据传递给工作流
        formData.forEach(localSdo::putProperty);
        // 将业务流程信息传递给工作流
        localSdo.putProperty("processAction", ProcessAction.SAVE);
        localSdo.putProperty("procUnitId", "Apply");
        localSdo.setOperator(operator);
        return localSdo;
    }
}
