package com.huigou.topsun.sap.costReimbursement.application.impl;


import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.topsun.sap.common.DefaultHttpClient;
import com.huigou.topsun.sap.common.serializer.MessageCodec;
import com.huigou.topsun.sap.costReimbursement.application.SapCostReimbursementApplication;
import com.huigou.topsun.sap.costReimbursement.domain.SapCostReimbursement;
import com.huigou.topsun.sap.costReimbursement.domain.SapCostReimbursementDetail;
import com.huigou.topsun.sap.costReimbursement.domain.query.SapCostReimbursementQueryRequest;
import com.huigou.topsun.sap.costReimbursement.repostiory.SapCostReimbursementDetailRepository;
import com.huigou.topsun.sap.costReimbursement.repostiory.SapCostReimbursementRepository;
import com.huigou.topsun.sap.statsIndicators.domain.SapStatsIndicators;
import com.huigou.topsun.sap.statsIndicators.domain.SapStatsIndicatorsDetail;
import com.huigou.topsun.sap.statsIndicators.domain.SapStatsIndicatorsResult;
import com.huigou.topsun.sap.statsIndicators.domain.query.SapStatsIndicatorsQueryRequest;
import com.huigou.topsun.util.MyBaseUtil;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.util.ClassHelper;
import com.huigou.util.DateUtil;
import com.huigou.util.SDO;
import lombok.SneakyThrows;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.io.IOException;
import java.util.*;

@Service("sapCostReimbursementApplication")
public class SapCostReimbursementApplicationImpl extends FlowBroker implements SapCostReimbursementApplication {

    private final static Logger LOG = LoggerFactory.getLogger(SapCostReimbursementApplicationImpl.class);
    @Autowired
    private SapCostReimbursementDetailRepository costReimbursementDetailRepository;
    @Autowired
    private SapCostReimbursementRepository costReimbursementRepository;


    @Autowired
    DefaultHttpClient defaultHttpClient;
//    @Autowired
//    MessageCodec messageCodec;


    @SneakyThrows
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();
        SDO sdo = this.getBizAndApprovalData();
        SapCostReimbursement sapCostReimbursement = getBizEntity(SapCostReimbursement.class);
        if (sapCostReimbursement.isNew()) {
            sapCostReimbursement.setStatusId(BizBillStatus.APPLYING.getId());
        } else {
            sapCostReimbursement = (SapCostReimbursement) commonDomainService.loadAndFillinProperties(sapCostReimbursement);
        }
        sapCostReimbursement = costReimbursementRepository.save(sapCostReimbursement);


        String CRDetailJSON = sdo.getString("CRDetail");
        ObjectMapper objectMapper = new ObjectMapper();
        List<SapCostReimbursementDetail> CRDetailAll = costReimbursementDetailRepository.findAllByReimbursementBaseInfoId(sapCostReimbursement.getId());
        if (!CRDetailAll.isEmpty()) {
            costReimbursementDetailRepository.delete(CRDetailAll);
        }
        List<SapCostReimbursementDetail> CRDetailList = objectMapper.readValue(CRDetailJSON, new TypeReference<List<SapCostReimbursementDetail>>() {
        });
        for (SapCostReimbursementDetail item : CRDetailList) {
            item.setReimbursementBaseInfoId(sapCostReimbursement.getId());
        }
        costReimbursementDetailRepository.save(CRDetailList);
        return sapCostReimbursement.getId();
    }


    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        // 返回业务数据给流程实例，
        return ClassHelper.toMap(costReimbursementRepository.getOne(bizId));
    }


    /**
     * 流程实例结束
     *
     * @param delegateExecution
     */
    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        BizBillStatus status = approvePassed() ? BizBillStatus.COMPLETED : BizBillStatus.ABORTED;
        SapCostReimbursement sapCostReimbursement = costReimbursementRepository.findOne(bizId);
        sapCostReimbursement.setStatusId(status.getId());
        costReimbursementRepository.save(sapCostReimbursement);
        //审核完成、远程调用sap接口
        sendSapDate(sapCostReimbursement);
    }

    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        SapCostReimbursement sapCostReimbursement = costReimbursementRepository.findOne(bizId);
        sapCostReimbursement.setStatusId(BizBillStatus.ABORTED.getId());
        costReimbursementRepository.save(sapCostReimbursement);
    }

    /**
     * @param delegateTask
     * @param destActivityId 回退到的目标节点id
     */
    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        super.onBack(delegateTask, destActivityId);
        if ("Apply".equalsIgnoreCase(destActivityId)) {
            String bizId = delegateTask.getExecution().getProcessBusinessKey();
            SapCostReimbursement sapCostReimbursement = costReimbursementRepository.findOne(bizId);
            sapCostReimbursement.setStatusId(BizBillStatus.APPLYING.getId());
            costReimbursementRepository.save(sapCostReimbursement);
        }
    }

    @Override
    public SapCostReimbursement loadSapCostReimbursementById(String id) {
        return costReimbursementRepository.findOne(id);
    }

    @Override
    public Map<String, Object> slicedSapCostReimbursementQuery(SapCostReimbursementQueryRequest query) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "slicedSapCostReimbursementQuery");
        return this.sqlExecutorDao.executeSlicedQuery(queryDescriptor, query);
    }


    @Override
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void deleteSapCostReimbursementById(String id) {
        SapCostReimbursement costReimbursement = costReimbursementRepository.findOne(id);
        List<SapCostReimbursementDetail> all = costReimbursementDetailRepository.findAllByReimbursementBaseInfoId(id);
        costReimbursementDetailRepository.delete(all);
        costReimbursementRepository.delete(costReimbursement);
    }

    @Override
    @Transactional
    public void deleteSapCostReimbursementByIds(List<String> ids) {
        ids.forEach(this::deleteSapCostReimbursementById);
    }

    /**
     * 远程调用sap接口，传递数据
     */
    public void sendSapDate(SapCostReimbursement sapCostReimbursement) {
        if (sapCostReimbursement.getStatusId() == BizBillStatus.APPLYING.getId()) {
            throw new RuntimeException("本数据未经审批完成！");
        }
        ObjectMapper objectMapper = new ObjectMapper();

        List<SapCostReimbursementDetail> all = costReimbursementDetailRepository.findAllByReimbursementBaseInfoId(sapCostReimbursement.getId());
        List<Object> paramList = new ArrayList<>();
        List<Map<String, Object>> itemList = new ArrayList<>();
        Map<String, Object> paramMap = new HashMap<>();
        new ArrayList<>();

        Map<String, Object> map = MyBaseUtil.convertToMap(sapCostReimbursement, true);
        map.remove("ZEP_FTYPE");
        map.remove("ZEP_NO");
        if (sapCostReimbursement.getZepFcrdate() != null){
            String dateFormat = DateUtil.getDateFormat(sapCostReimbursement.getZepFcrdate(), "yyyyMMdd");
            map.put("ZEP_FCRDATE",dateFormat);
        }

        // 封装参数
        for (SapCostReimbursementDetail costReimbursementDetail : all) {

            Map<String, Object> detailMap = MyBaseUtil.convertToMap(costReimbursementDetail, true);
            if (costReimbursementDetail.getZepFpldate() != null){
                String dateFormat = DateUtil.getDateFormat(costReimbursementDetail.getZepFpldate(), "yyyyMMdd");
                detailMap.put("ZEP_FPLDATE",dateFormat);
            }
            String budat = DateUtil.getDateFormat(new Date(), "yyyyMMdd");
            detailMap.put("BUDAT",budat);
            detailMap.put("BLDAT",budat);
            detailMap.putAll(map);
            itemList.add(detailMap);
            costReimbursementDetail.setBudat(new Date());
            costReimbursementDetail.setBldat(new Date());
            costReimbursementDetailRepository.save(costReimbursementDetail);
        }
        paramMap.put("ITEM", itemList);
        paramMap.put("MANDT", "300");
        paramMap.put("ZEP_FTYPE", sapCostReimbursement.getZepFtype());
        paramMap.put("ZEP_NO", sapCostReimbursement.getBillCode());
        paramList.add(paramMap);

        String url = "cud_fi012/cud_fi012";
        try {
            String result = defaultHttpClient.execute(paramList, url);
            List<Map<String, Object>> resultList = objectMapper.readValue(result, new TypeReference<List<Map<String, Object>>>() {
            });

            StringBuffer errorMsg = new StringBuffer();
            for (Map<String, Object> resultMap : resultList) {
                if (!"S".equals(resultMap.get("TYPE"))) {
                    errorMsg.append(resultMap.get("MESSAGE"));
                }
            }
            if (errorMsg.length()==0) {
                Map<String, Object> resultMap = resultList.get(0);
                sapCostReimbursement.setMsgId((String) resultMap.get("ID"));
                sapCostReimbursement.setNumber((String) resultMap.get("NUMBER"));
                sapCostReimbursement.setMsgType((String) resultMap.get("TYPE"));
                sapCostReimbursement.setLogNo((String) resultMap.get("LOG_NO"));
                sapCostReimbursement.setLogMsgNo((String) resultMap.get("LOG_MSG_NO"));
                sapCostReimbursement.setMessage((String) resultMap.get("MESSAGE"));
                sapCostReimbursement.setMessageV1((String) resultMap.get("MESSAGE_V1"));
                sapCostReimbursement.setMessageV2((String) resultMap.get("MESSAGE_V1"));
                sapCostReimbursement.setMessageV3((String) resultMap.get("MESSAGE_V1"));
                sapCostReimbursement.setMessageV4((String) resultMap.get("MESSAGE_V1"));
                this.costReimbursementRepository.save(sapCostReimbursement);
            } else {
                LOG.error("数据传输失败！error: {}", errorMsg);
                throw new RuntimeException("数据传输失败，请检查后重试！error:" + errorMsg);
            }
        } catch (IOException e) {
            LOG.error("sap服务器无响应请稍后手动重试！error: {}", e.getMessage());
            throw new RuntimeException("sap服务器无响应请稍后手动重试！error:" + e.getMessage());
        }
    }

    public static void main(String[] args) throws JsonProcessingException {
//        post.setHeader("Content-type", "application/json;charset=utf-8");

        String usernameAndPassword = "rest_user:Bsn123456";
        System.out.println("Authorization");
        System.out.println("Basic " + Base64.getEncoder().encodeToString(usernameAndPassword.getBytes()));
    }

}