package com.huigou.topsun.sap.nonProdApply.application.impl;


import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.huigou.topsun.base.bsnMessage.appliction.BsnMessageApplication;
import com.huigou.topsun.base.bsnMessage.domain.*;
import com.huigou.topsun.sap.nonProdApply.application.NonProdApplyDeptApplication;

import com.huigou.topsun.sap.nonProdApply.domain.SapNonProdApplyDept;
import com.huigou.topsun.sap.nonProdApply.domain.vo.*;
import com.huigou.topsun.sap.nonProdApply.repository.NonProdApplyDeptRepository;
import com.huigou.uasp.bmp.common.application.BaseApplication;

import com.huigou.util.ClassHelper;
import com.huigou.util.SDO;
import com.huigou.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class NonProdApplyDeptApplicationImpl extends BaseApplication implements NonProdApplyDeptApplication {

    @Autowired
    NonProdApplyDeptRepository nonProdApplyDeptRepository;
    @Autowired
    BsnMessageApplication bsnMessageApplication;

    @Override
    public Map<String,Object> saveSapNonProdApplyDept(SDO sdo){
        Map<String,Object> map = new HashMap<>();
        String businessCode=sdo.getString("businessCode");
        String genericName=sdo.getString("genericName");
        Integer active=sdo.getInteger("active");
        String meId=sdo.getString("meId");
        NonProdApplyDeptVo deptVo=null;
        switch (businessCode) {
            case "K":
                deptVo = sdo.toObject(NonProdApplyBasicVo.class);
                break;
            case "C":
                deptVo = sdo.toObject(NonProdApplyClassVo.class);
                break;
            case "V":
                deptVo = sdo.toObject(NonProdApplySaleVo.class);
                break;
            case "E":
                deptVo = sdo.toObject(NonProdApplyPurchaseVo.class);
                break;
            case "D":
                deptVo = sdo.toObject(NonProdApplyMRPVo.class);
                break;
            case "A":
                deptVo = sdo.toObject(NonProdApplyProduceVo.class);
                break;
            case "L":
                deptVo = sdo.toObject(NonProdApplyStoreVo.class);
                break;
            case "S":
                deptVo = sdo.toObject(NonProdApplyWarehouseVo.class);
                break;
            case "Q":
                deptVo = sdo.toObject(NonProdApplyMassVo.class);
                break;
            case "B":
                deptVo = sdo.toObject(NonProdApplyFinancialVo.class);
                break;
            case "G":
                deptVo = sdo.toObject(NonProdApplyCostVo.class);
                break;
            default:
                deptVo = sdo.toObject(NonProdApplyBasicVo.class);
        }
       // SapNonProdApplyDept nonProdApplyDeptObj = sdo.toObject(SapNonProdApplyDept.class);
        //nonProdApplyDeptObj.setBaseInfoId(sdo.getId());
        deptVo.setBaseInfoId(sdo.getId());
        SapNonProdApplyDept nonProdApplyDept=nonProdApplyDeptRepository.findByBaseInfoId(sdo.getId());
        if (nonProdApplyDept==null){
            nonProdApplyDept=new SapNonProdApplyDept();
        }
        ClassHelper.copyProperties(deptVo,nonProdApplyDept);

        nonProdApplyDept = this.save(nonProdApplyDept);
        String applyDeptId=sdo.getString("applyDeptId");

        if (StringUtil.isBlank(businessCode)) { //业务类型为空 ，表示是基本信息进入的  才保存分发信息
            //保存部门分发任务
            //获取分发的部门与人员信息
            ObjectMapper objectMapper = new ObjectMapper();
            String deptStaffJSON = sdo.getString("deptStaff");
            List<Map<String, String>> deptStaffList = null;
            try {
                deptStaffList = objectMapper.readValue(deptStaffJSON, new TypeReference<List<Map<String, String>>>() {
                });
            } catch (JsonProcessingException e) {
                throw new RuntimeException("JSON转换失败：" + e);
            }
            saveDeptStaff(deptStaffList, nonProdApplyDept.getApplyDeptId(),genericName);
        }
        //if (active == 1){//各部门提交后，隐藏消息
            //bsnMessageApplication.submitTask(meId);
        //}
        map.put("applyDeptId",nonProdApplyDept.getApplyDeptId());
        return map;
    }

    @Override
    public SapNonProdApplyDept save(SapNonProdApplyDept sapNonProdApplyDept){
        return nonProdApplyDeptRepository.save(sapNonProdApplyDept);
    }

    public void saveDeptStaff(List<Map<String, String>> deptStaffList,String bussinessId,String genericName){
        //封装分发数据
        BsnMessageSponsor bsnMessageSponsor = new BsnMessageSponsor();
        bsnMessageSponsor.setTaskSponsor(this.getOperator().getUserId());
        bsnMessageSponsor.setTitle(genericName);//通用品名
        bsnMessageSponsor.setExecutorUrl("sapNonProdApplyNext/fillInNonProdApplyDept.do");
        bsnMessageSponsor.setBusinessId(bussinessId);
        bsnMessageSponsor.setCreateDate(new Date());
        bsnMessageSponsor.setStatus(0);

        for (Map<String, String> stringMap : deptStaffList) {
            BsnMessageExecute messageExecute = new BsnMessageExecute();
            String userCode = stringMap.get("userCode");
            String deptName = stringMap.get("deptName");
            String deptCode = stringMap.get("deptCode");
            String[] split = userCode.split("@");
            messageExecute.setTaskExecute(split[0]);
            messageExecute.setBusinessCode(deptCode);
            messageExecute.setActive(ActiveStatus.UNFINISHED.getValue());
            messageExecute.setUpdateDate(new Date());
            messageExecute.setTaskDescribe(deptName+"部门数据维护");
            bsnMessageSponsor.addExecute(messageExecute);
        }
        //保存任务
        bsnMessageApplication.keepTasks(bsnMessageSponsor);

    }
    /**
     * 发布任务
     * @param sdo
     */
    @Override
    public void distributeSapNonProdApplyDept(SDO sdo) {
        //将状态为保存未分发的全部修改为分发未完成
        Map<String, Object> map = this.saveSapNonProdApplyDept(sdo);
        //获取分发的部门与人员信息
        ObjectMapper objectMapper = new ObjectMapper();
        String deptStaffJSON = sdo.getString("deptStaff");
        List<Map<String, String>> deptStaffList = null;
        try {
            deptStaffList = objectMapper.readValue(deptStaffJSON, new TypeReference<List<Map<String, String>>>() {});
        } catch (JsonProcessingException e) {
            throw new RuntimeException("JSON转换失败："+e);
        }
        //保存分发任务
       // saveDeptStaff(deptStaffList,map.get("applyDeptId").toString());

         //发布任务
        bsnMessageApplication.publishTasks((String) map.get("applyDeptId"));
    }

    @Override
    public SapNonProdApplyDept getNonProdApplyDeptById(String id) {
        return nonProdApplyDeptRepository.findByApplyDeptId(id);
    }

    @Override
    public SapNonProdApplyDept getNonProdApplyDeptByBaseInfoId(String baseInfoId) {
        return nonProdApplyDeptRepository.findByBaseInfoId(baseInfoId);
    }

    @Override
    public void rejectNonProdApplyDept(SDO sdo) {
        String meId = sdo.getString("meId");
        Map<String,Object> map = new HashMap<>();
        String auditDept = sdo.getString("auditDept");
        List<String> deptList = Arrays.asList(auditDept.split(","));
        for (String key : deptList) {
            map.put(key,sdo.getString(key));
        }
        bsnMessageApplication.rejectTasksByMeId(map,meId);
    }
}
