package com.huigou.topsun.sap.nonProducePick.application.impl;

import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.sap.nonProducePick.application.NonProducePickApplication;
import com.huigou.topsun.sap.nonProducePick.application.NonProducePickItemApplication;
import com.huigou.topsun.sap.nonProducePick.domain.NonProducePick;
import com.huigou.topsun.sap.nonProducePick.domain.NonProducePickItem;
import com.huigou.topsun.sap.nonProducePick.domain.query.NonProducePickQueryRequest;
import com.huigou.topsun.sap.nonProducePick.repository.NonProducePickRepository;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.util.ClassHelper;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @Auther: xin.lu
 * @Date: 2024/03/13/9:45
 * @Description:
 */
@Service("nonProducePickApplication")
public class NonProducePickApplicationImpl extends FlowBroker implements NonProducePickApplication {
    @Autowired
    private NonProducePickRepository nonProducePickRepository;
    @Autowired
    private NonProducePickItemApplication nonProducePickItemApplication;
    @Override
    public NonProducePick findNonProducePickById(String id) {
        return nonProducePickRepository.findOne(id);
    }

    @Override
    public void deleteNonProducePickById(List<String> ids) {
        ids.forEach(id->{
            nonProducePickRepository.delete(id);
        });
    }

    @Override
    public NonProducePick saveNonProducePick(NonProducePick nonProducePick) {
        return nonProducePickRepository.save(nonProducePick);
    }

    @Override
    public Map<String, Object> slicedNonProducePickList(NonProducePickQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "nonProducePick");
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        Map<String, Object> map = this.sqlExecutorDao.executeSlicedQuery(queryModel);
        return map;
    }

    @Override
    public void sendNonProducePickData(NonProducePick nonProducePick) {

    }

    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();
        NonProducePick nonProducePick = getBizEntity(NonProducePick.class);
        if (nonProducePick.isNew()) {
            nonProducePick.setStatusId(BizBillStatus.APPLYING.getId());
        } else {
            nonProducePick = (NonProducePick) commonDomainService.loadAndFillinProperties(nonProducePick);
        }
        nonProducePick = nonProducePickRepository.save(nonProducePick);
        List<NonProducePickItem> nonProducePickItems = getBizEntities(NonProducePickItem.class, "items");
        nonProducePickItemApplication.saveNonProducePickItems(nonProducePick.getId(),nonProducePickItems);
        return nonProducePick.getId();
    }

    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        // 返回业务数据给流程实例，
        return ClassHelper.toMap(nonProducePickRepository.getOne(bizId));
    }

    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        BizBillStatus status = approvePassed() ? BizBillStatus.COMPLETED : BizBillStatus.ABORTED;
        NonProducePick nonProducePick = nonProducePickRepository.findOne(bizId);
        nonProducePick.setStatusId(status.getId());
        nonProducePickRepository.save(nonProducePick);
        /*try {
            String url = SystemCache.getParameter("sap.purchase.url", String.class);
            String execute = httpClient.execute(nonProducePick, url);
            List<SapResult> resultList = JSONObject.parseArray(execute, SapResult.class);
            SapResult sapResult = resultList.get(0);
            if (!"S".equals(sapResult.getTYPE())){
                throw new RuntimeException("数据传输失败，"+ sapResult.getMESSAGE());
            }
            nonProducePickRepository.save(nonProducePick);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }*/
    }

    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        NonProducePick nonProducePick = nonProducePickRepository.findOne(bizId);
        nonProducePick.setStatusId(BizBillStatus.ABORTED.getId());
        nonProducePickRepository.save(nonProducePick);
    }

    /**
     * @param delegateTask
     * @param destActivityId 回退到的目标节点id
     */
    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        super.onBack(delegateTask, destActivityId);
        if ("Apply".equalsIgnoreCase(destActivityId)) {
            String bizId = delegateTask.getExecution().getProcessBusinessKey();
            NonProducePick nonProducePick = nonProducePickRepository.findOne(bizId);
            nonProducePick.setStatusId(BizBillStatus.APPLYING.getId());
            nonProducePickRepository.save(nonProducePick);
        }
    }
}
