package com.huigou.topsun.sap.purchase.application.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.huigou.cache.SystemCache;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.sap.common.HttpClient;
import com.huigou.topsun.sap.purchase.application.SapPurchaseApplication;
import com.huigou.topsun.sap.purchase.application.SapPurchaseItemApplication;
import com.huigou.topsun.sap.purchase.domain.SapPurchase;
import com.huigou.topsun.sap.purchase.domain.SapPurchaseItem;
import com.huigou.topsun.sap.purchase.domain.query.SapPurchaseQueryRequest;
import com.huigou.topsun.sap.purchase.domain.vo.SapPurchaseItemVo;
import com.huigou.topsun.sap.purchase.domain.vo.SapPurchaseVo;
import com.huigou.topsun.sap.purchase.repository.SapPurchaseRepository;
import com.huigou.topsun.sap.purchaseInfoRecord.domain.SapResult;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.util.ClassHelper;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Auther: xin.lu
 * @Date: 2023/12/27/10:38
 * @Description:
 */
@Service("sapPurchaseApplication")
public class SapPurchaseApplicationImpl extends FlowBroker implements SapPurchaseApplication {
    @Resource
    private SapPurchaseRepository sapPurchaseRepository;
    @Resource
    private SapPurchaseItemApplication sapPurchaseItemApplication;
    @Resource
    private HttpClient httpClient;
    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();
        SapPurchase sapPurchase = getBizEntity(SapPurchase.class);
        if (sapPurchase.isNew()) {
            sapPurchase.setStatusId(BizBillStatus.APPLYING.getId());
        } else {
            sapPurchase = (SapPurchase) commonDomainService.loadAndFillinProperties(sapPurchase);
        }
        sapPurchase = sapPurchaseRepository.save(sapPurchase);
        List<SapPurchaseItem> sapPurchaseItems = getBizEntities(SapPurchaseItem.class, "sapPurchaseItems");
        sapPurchaseItemApplication.saveSapPurchaseItems(sapPurchase.getId(),sapPurchaseItems);

        /*List<SapPurchaseVo> purchaseVoList = this.getSapPurchaseVoJson(sapPurchase);
        try {
            String url = SystemCache.getParameter("sap.purchase.url", String.class);
            String execute = httpClient.execute(purchaseVoList, url);
            System.out.println("返回结果："+execute);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }*/
        return sapPurchase.getId();
    }



    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        // 返回业务数据给流程实例，
        return ClassHelper.toMap(sapPurchaseRepository.getOne(bizId));
    }

    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        BizBillStatus status = approvePassed() ? BizBillStatus.COMPLETED : BizBillStatus.ABORTED;
        SapPurchase sapPurchase = sapPurchaseRepository.findOne(bizId);
        sapPurchase.setStatusId(status.getId());
        sapPurchase.setFrgdt(new Date());

        SapPurchaseVo sapPurchaseVo = this.getSapPurchaseVoJson(sapPurchase);
        try {
            String url = SystemCache.getParameter("sap.purchase.url", String.class);
            String execute = httpClient.execute(sapPurchaseVo, url);
            List<SapResult> resultList = JSONObject.parseArray(execute, SapResult.class);
            SapResult sapResult = resultList.get(0);
            if (!"S".equals(sapResult.getTYPE())){
                throw new RuntimeException("数据传输失败，"+ sapResult.getMESSAGE());
            }
            sapPurchase.setBanfn(sapResult.getMESSAGE_V1());//SAP采购申请编号
            sapPurchaseRepository.save(sapPurchase);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 转换成SAP JSON格式数据
     * @param sapPurchase
     * @return
     */
    public SapPurchaseVo getSapPurchaseVoJson(SapPurchase sapPurchase){
        List<SapPurchaseItem> sapPurchaseItemList = this.sapPurchaseItemApplication.findBySapPurchaseId(sapPurchase.getId());
        List<SapPurchaseItemVo> sapPurchaseItemVos = new ArrayList<>();
        SapPurchaseVo sapPurchaseVo = new SapPurchaseVo();
        sapPurchaseVo.setBsart(sapPurchase.getBsart());
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd");
        sapPurchaseItemList.forEach(sapPurchaseItem -> {
            SapPurchaseItemVo sapPurchaseItemVo = new SapPurchaseItemVo();
            BeanUtil.copyProperties(sapPurchaseItem,sapPurchaseItemVo);
            sapPurchaseItemVo.setBillCode(sapPurchase.getBillCode());
            sapPurchaseItemVo.setErnam(sapPurchase.getPersonMemberName());
            if (sapPurchaseItem.getBadat() != null){
                String format = dateFormat.format(sapPurchaseItem.getBadat());
                sapPurchaseItemVo.setBadat(format);
            }
            if (sapPurchaseItem.getErdat()!=null){
                String format = dateFormat.format(sapPurchaseItem.getErdat());
                sapPurchaseItemVo.setErdat(format);
            }
            if (sapPurchase.getFrgdt()!=null){
                String format = dateFormat.format(sapPurchase.getFrgdt());
                sapPurchaseItemVo.setFrgdt(format);
            }
            if (sapPurchaseItem.getLfdat()!=null){
                String format = dateFormat.format(sapPurchaseItem.getLfdat());
                sapPurchaseItemVo.setLfdat(format);
            }
            sapPurchaseItemVos.add(sapPurchaseItemVo);
        });
        sapPurchaseVo.setItemVos(sapPurchaseItemVos);
        /*ObjectMapper objectMapper = new ObjectMapper();
        String returnJson = null;
        try {
            returnJson = objectMapper.writeValueAsString(sapPurchaseVoList);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }*/
        return sapPurchaseVo;
    }

    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        SapPurchase sapPurchase = sapPurchaseRepository.findOne(bizId);
        sapPurchase.setStatusId(BizBillStatus.ABORTED.getId());
        sapPurchaseRepository.save(sapPurchase);
    }

    /**
     * @param delegateTask
     * @param destActivityId 回退到的目标节点id
     */
    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        super.onBack(delegateTask, destActivityId);
        if ("Apply".equalsIgnoreCase(destActivityId)) {
            String bizId = delegateTask.getExecution().getProcessBusinessKey();
            SapPurchase sapPurchase = sapPurchaseRepository.findOne(bizId);
            sapPurchase.setStatusId(BizBillStatus.APPLYING.getId());
            sapPurchaseRepository.save(sapPurchase);
        }
    }

    @Override
    public Map<String, Object> slicedSapPurchaseList(SapPurchaseQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "sapPurchaseList");
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        queryModel.putDictionary("status", BizBillStatus.getMap());
        return this.sqlExecutorDao.executeSlicedQuery(queryModel);
    }

    @Override
    public SapPurchase loadSapPurchase(String id) {
        return sapPurchaseRepository.findOne(id);
    }
}
