package com.huigou.topsun.sap.purchaseInfoRecord.application.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSONObject;
import com.huigou.cache.SystemCache;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.sap.common.HttpClient;
import com.huigou.topsun.sap.purchaseInfoRecord.application.SapPurchaseInfoRecordApplication;
import com.huigou.topsun.sap.purchaseInfoRecord.application.SapPurchaseInfoRecordItemApplication;
import com.huigou.topsun.sap.purchaseInfoRecord.domain.SapPurchaseInfoRecord;
import com.huigou.topsun.sap.purchaseInfoRecord.domain.SapPurchaseInfoRecordItem;
import com.huigou.topsun.sap.purchaseInfoRecord.domain.SapResult;
import com.huigou.topsun.sap.purchaseInfoRecord.domain.query.SapPurchaseInfoRecordQueryRequest;
import com.huigou.topsun.sap.purchaseInfoRecord.domain.vo.SapPurchaseInfoRecordItemVo;
import com.huigou.topsun.sap.purchaseInfoRecord.repository.SapPurchaseInfoRecordRepository;
import com.huigou.topsun.sap.statsIndicators.domain.SapStatsIndicatorsResult;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.util.ClassHelper;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Auther: xin.lu
 * @Date: 2024/02/28/15:40
 * @Description:
 */
@Service("sapPurchaseInfoRecordApplication")
public class SapPurchaseInfoRecordApplicationImpl extends FlowBroker implements SapPurchaseInfoRecordApplication {

    @Autowired
    private SapPurchaseInfoRecordRepository sapPurchaseInfoRecordRepository;
    @Autowired
    private SapPurchaseInfoRecordItemApplication sapPurchaseInfoRecordItemApplication;
    @Resource
    private HttpClient httpClient;
    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();
        SapPurchaseInfoRecord sapPurchaseInfoRecord = getBizEntity(SapPurchaseInfoRecord.class);
        if (sapPurchaseInfoRecord.isNew()) {
            sapPurchaseInfoRecord.setStatusId(BizBillStatus.APPLYING.getId());
        } else {
            sapPurchaseInfoRecord = (SapPurchaseInfoRecord) commonDomainService.loadAndFillinProperties(sapPurchaseInfoRecord);
        }
        sapPurchaseInfoRecord = sapPurchaseInfoRecordRepository.save(sapPurchaseInfoRecord);
        List<SapPurchaseInfoRecordItem> sapPurchaseInfoRecordItems = getBizEntities(SapPurchaseInfoRecordItem.class, "sapPurchaseInfoRecordItems");
        sapPurchaseInfoRecordItemApplication.saveSapPurchaseInfoRecordItem(sapPurchaseInfoRecord.getId(),sapPurchaseInfoRecordItems);

        return sapPurchaseInfoRecord.getId();
    }



    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        // 返回业务数据给流程实例，
        return ClassHelper.toMap(sapPurchaseInfoRecordRepository.getOne(bizId));
    }

    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        BizBillStatus status = approvePassed() ? BizBillStatus.COMPLETED : BizBillStatus.ABORTED;
        SapPurchaseInfoRecord sapPurchaseInfoRecord = sapPurchaseInfoRecordRepository.findOne(bizId);
        sapPurchaseInfoRecord.setStatusId(status.getId());
        sapPurchaseInfoRecordRepository.save(sapPurchaseInfoRecord);

        this.getSapPurchaseInfoRecordItemVo(bizId);
    }

    public void getSapPurchaseInfoRecordItemVo(String bizId){
        List<SapPurchaseInfoRecordItem> purchaseInfoRecordItemList = sapPurchaseInfoRecordItemApplication.findBySapPurchaseInfoRecordId(bizId);
        purchaseInfoRecordItemList.forEach(sapPurchaseInfoRecordItem -> {
            SapPurchaseInfoRecordItemVo sapPurchaseInfoRecordItemVo = new SapPurchaseInfoRecordItemVo();
            BeanUtil.copyProperties(sapPurchaseInfoRecordItem,sapPurchaseInfoRecordItemVo);
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd");
            if (sapPurchaseInfoRecordItem.getDatab() !=null){
                String datab = dateFormat.format(sapPurchaseInfoRecordItem.getDatab());
                sapPurchaseInfoRecordItemVo.setDatab(datab);
            }
            if (sapPurchaseInfoRecordItem.getDatbi() != null){
                String datbi = dateFormat.format(sapPurchaseInfoRecordItem.getDatbi());
                sapPurchaseInfoRecordItemVo.setDatbi(datbi);
            }
            try {
                String url = SystemCache.getParameter("sap.PurchaseInfoRecord.url", String.class);
                String execute = httpClient.execute(sapPurchaseInfoRecordItemVo, url);
                List<SapResult> resultList = JSONObject.parseArray(execute, SapResult.class);
                SapResult sapResult = resultList.get(0);
                if (!"S".equals(sapResult.getTYPE())){
                    throw new RuntimeException("数据传输失败，"+ sapResult.getMESSAGE());
                }
                sapPurchaseInfoRecordItem.setTYPE(sapResult.getTYPE());
                sapPurchaseInfoRecordItem.setMESSAGE(sapResult.getMESSAGE());
                sapPurchaseInfoRecordItemApplication.save(sapPurchaseInfoRecordItem);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        });
    }

    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        SapPurchaseInfoRecord sapPurchaseInfoRecord = sapPurchaseInfoRecordRepository.findOne(bizId);
        sapPurchaseInfoRecord.setStatusId(BizBillStatus.ABORTED.getId());
        sapPurchaseInfoRecordRepository.save(sapPurchaseInfoRecord);
    }

    /**
     * @param delegateTask
     * @param destActivityId 回退到的目标节点id
     */
    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        super.onBack(delegateTask, destActivityId);
        if ("Apply".equalsIgnoreCase(destActivityId)) {
            String bizId = delegateTask.getExecution().getProcessBusinessKey();
            SapPurchaseInfoRecord sapPurchaseInfoRecord = sapPurchaseInfoRecordRepository.findOne(bizId);
            sapPurchaseInfoRecord.setStatusId(BizBillStatus.APPLYING.getId());
            sapPurchaseInfoRecordRepository.save(sapPurchaseInfoRecord);
        }
    }

    @Override
    public Map<String, Object> slicedSapPurchaseInfoRecordList(SapPurchaseInfoRecordQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "sapPurchaseInfoRecordList");
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        queryModel.putDictionary("status", BizBillStatus.getMap());
        return this.sqlExecutorDao.executeSlicedQuery(queryModel);
    }

    @Override
    public SapPurchaseInfoRecord loadSapPurchaseInfoRecord(String id) {
        return sapPurchaseInfoRecordRepository.findOne(id);
    }

}
