package com.huigou.topsun.sap.purchaseOrder.application.impl;

import cn.hutool.core.bean.BeanUtil;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.huigou.context.Operator;
import com.huigou.context.OrgUnit;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.sap.common.DefaultHttpClient;
import com.huigou.topsun.sap.purchaseApproval.application.impl.SapPurchaseApprovalApplicationImpl;
import com.huigou.topsun.sap.purchaseOrder.application.SapPurchaseOrderApplication;
import com.huigou.topsun.sap.purchaseOrder.application.SapPurchaseOrderItemApplication;
import com.huigou.topsun.sap.purchaseOrder.domain.SapPurchaseOrder;
import com.huigou.topsun.sap.purchaseOrder.domain.SapPurchaseOrderItem;
import com.huigou.topsun.sap.purchaseOrder.domain.query.SapPurchaseOrderQueryRequest;
import com.huigou.topsun.sap.purchaseOrder.domain.vo.SapPurchaseOrderDto;
import com.huigou.topsun.sap.purchaseOrder.domain.vo.SapPurchaseOrderItemVo;
import com.huigou.topsun.sap.purchaseOrder.domain.vo.SapPurchaseOrderVo;
import com.huigou.topsun.sap.purchaseOrder.repository.SapPurchaseOrderRepository;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bmp.operator.OperatorApplication;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.util.ClassHelper;
import com.huigou.util.SDO;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

/**
 * @Auther: xin.lu
 * @Date: 2024/01/09/18:59
 * @Description:
 */
@Service("sapPurchaseOrderApplication")
public class SapPurchaseOrderApplicationImpl extends FlowBroker implements SapPurchaseOrderApplication {
    @Resource
    private SapPurchaseOrderRepository sapPurchaseOrderRepository;
    @Resource
    private SapPurchaseOrderItemApplication sapPurchaseOrderItemApplication;
    @Resource
    private OperatorApplication operatorApplication;
    @Autowired
    DefaultHttpClient defaultHttpClient;
    @Autowired
    SapPurchaseApprovalApplicationImpl sapPurchaseApprovalApplication;

    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();
        SDO sdo = this.getBizAndApprovalData();
        SapPurchaseOrder sapPurchaseOrder = getBizEntity(SapPurchaseOrder.class);
        if (sapPurchaseOrder.isNew()) {
            sapPurchaseOrder.setStatusId(BizBillStatus.APPLYING.getId());
        } else {
            sapPurchaseOrder = (SapPurchaseOrder) commonDomainService.loadAndFillinProperties(sapPurchaseOrder);
        }
        sapPurchaseOrder = sapPurchaseOrderRepository.save(sapPurchaseOrder);
//        SapPurchaseOrderItem sapPurchaseOrderItem = getBizEntity(SapPurchaseOrderItem.class);
//        if(sapPurchaseOrderItem!=null){
//            sapPurchaseOrderItem.setPurchaseOrderId(sapPurchaseOrder.getId());
//            sapPurchaseOrderItemApplication.saveSapPurchaseOrderItem(sapPurchaseOrderItem);
//        }
        return sapPurchaseOrder.getId();
    }


    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        // 返回业务数据给流程实例
        return ClassHelper.toMap(sapPurchaseOrderRepository.getOne(bizId));
    }

    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        BizBillStatus status = approvePassed() ? BizBillStatus.COMPLETED : BizBillStatus.ABORTED;
        SapPurchaseOrder sapPurchaseOrder = sapPurchaseOrderRepository.findOne(bizId);
        sapPurchaseOrder.setStatusId(status.getId());
        SapPurchaseOrder save = sapPurchaseOrderRepository.save(sapPurchaseOrder);
        this.sendSapDate(save, "Y");
    }


    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        SapPurchaseOrder sapPurchaseOrder = sapPurchaseOrderRepository.findOne(bizId);
        sapPurchaseOrder.setStatusId(BizBillStatus.ABORTED.getId());
        sapPurchaseOrderRepository.save(sapPurchaseOrder);
    }

    /**
     * @param delegateTask
     * @param destActivityId 回退到的目标节点id
     */
    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        super.onBack(delegateTask, destActivityId);
        if ("Apply".equalsIgnoreCase(destActivityId)) {
            String bizId = delegateTask.getExecution().getProcessBusinessKey();
            SapPurchaseOrder sapPurchaseOrder = sapPurchaseOrderRepository.findOne(bizId);
            sapPurchaseOrder.setStatusId(BizBillStatus.APPLYING.getId());
            SapPurchaseOrder save = sapPurchaseOrderRepository.save(sapPurchaseOrder);
            this.sendSapDate(save, "N");
            // 调用 onEnd 方法
//            DelegateExecution execution = delegateTask.getExecution();
//            onEnd(execution);
        }
    }

    @Override
    public Map<String, Object> slicedSapPurchaseOrderList(SapPurchaseOrderQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "sapPurchaseOrderList");
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        queryModel.putDictionary("status", BizBillStatus.getMap());
        return this.sqlExecutorDao.executeSlicedQuery(queryModel);
    }

    @Override
    public SapPurchaseOrder loadSapPurchaseOrder(String id) {
        return sapPurchaseOrderRepository.findOne(id);
    }

    @Override
    @Transactional
    public void saveSapPurchaseOrderDto(SapPurchaseOrderDto sapPurchaseOrderDto) {
        SapPurchaseOrderVo sapPurchaseOrderVo = sapPurchaseOrderDto.getLsHeader();
        SapPurchaseOrder purchaseOrder = sapPurchaseOrderRepository.findAllByEbeln(sapPurchaseOrderVo.getEbeln());
        String purchaseOrderId = "";
        //数据第一次接收，创建流程任务
        if (purchaseOrder == null) {
            SapPurchaseOrder sapPurchaseOrder = new SapPurchaseOrder();
            BeanUtil.copyProperties(sapPurchaseOrderVo, sapPurchaseOrder);
            //初始化用户信息
            String createdById = "5EEB2DC861694A65933CB4CE9A8D870A@C18E23AAAA6940E09DF7C122332C51E0";
            Operator operator = operatorApplication.createOperatorByPersonMemberId(createdById);
            sapPurchaseOrder.setDefaultValues(new OrgUnit(operator.getFullId(), operator.getFullName()));

            Map<String, Object> formData = BeanUtil.beanToMap(sapPurchaseOrder);
            String bizId = sapPurchaseApprovalApplication.startProcessInstance(SapPurchaseOrderApplication.PROCESS_DEFINITION_KEY, formData);
            sapPurchaseOrder.setId(bizId);
            //映射双方字段
            sapPurchaseOrder.setFillinDate(sapPurchaseOrderVo.getAedat());

            sapPurchaseOrder = this.sapPurchaseOrderRepository.save(sapPurchaseOrder);
            purchaseOrderId = sapPurchaseOrder.getId();
        } else {
            //数据第n次接收覆盖上一次数据
            BeanUtil.copyProperties(sapPurchaseOrderVo, purchaseOrder);
            this.sapPurchaseOrderRepository.save(purchaseOrder);

            List<SapPurchaseOrderItem> purchaseOrderItems = sapPurchaseOrderItemApplication.getPurchaseOrderItems(purchaseOrder.getId());
            sapPurchaseOrderItemApplication.deletePurchaseOrderItems(purchaseOrderItems);
            purchaseOrderId = purchaseOrder.getId();
        }

        List<SapPurchaseOrderItemVo> ltItems = sapPurchaseOrderDto.getLtItems();
        List<SapPurchaseOrderItem> purchaseOrderItems = new ArrayList<>();
        ltItems.forEach(approvalItemVo -> {
            SapPurchaseOrderItem sapPurchaseApprovalItem = new SapPurchaseOrderItem();
            BeanUtil.copyProperties(approvalItemVo, sapPurchaseApprovalItem);
            purchaseOrderItems.add(sapPurchaseApprovalItem);
        });
        sapPurchaseOrderItemApplication.saveSapPurchaseOrderItems(purchaseOrderId, purchaseOrderItems);
    }


    /**
     * 远程调用sap接口，传递数据
     */
    public void sendSapDate(SapPurchaseOrder sapPurchaseOrder, String flag) {
        Map<String, Object> paramItem = new HashMap<>();
        paramItem.put("ebeln", sapPurchaseOrder.getEbeln());
        paramItem.put("BEDNR", sapPurchaseOrder.getBillCode());
        paramItem.put("frgkz", "10");
        paramItem.put("flag", flag);
        List<Map<String, Object>> paramList = new ArrayList<>();
        paramList.add(paramItem);
        Map<String, Object> param = new HashMap<>();
        param.put("MODE", "PO");
        param.put("DATA", paramList);

        ObjectMapper objectMapper = new ObjectMapper();
        String url = "pass_approval/approval";
        try {
            String result = defaultHttpClient.execute(param, url);
            List<Map<String, Object>> resultList = objectMapper.readValue(result, new TypeReference<List<Map<String, Object>>>() {
            });
            Map<String, Object> resultMap = resultList.get(0);
            if ("S".equals(resultMap.get("TYPE"))) {
                sapPurchaseOrder.setMsgId((String) resultMap.get("ID"));
                sapPurchaseOrder.setNumber((String) resultMap.get("NUMBER"));
                sapPurchaseOrder.setMsgType((String) resultMap.get("TYPE"));
                sapPurchaseOrder.setLogNo((String) resultMap.get("LOG_NO"));
                sapPurchaseOrder.setLogMsgNo((String) resultMap.get("LOG_MSG_NO"));
                sapPurchaseOrder.setMessage((String) resultMap.get("MESSAGE"));
                sapPurchaseOrder.setMessageV1((String) resultMap.get("MESSAGE_V1"));
                sapPurchaseOrder.setMessageV2((String) resultMap.get("MESSAGE_V1"));
                sapPurchaseOrder.setMessageV3((String) resultMap.get("MESSAGE_V1"));
                sapPurchaseOrder.setMessageV4((String) resultMap.get("MESSAGE_V1"));
                this.sapPurchaseOrderRepository.save(sapPurchaseOrder);
            } else {
                throw new RuntimeException("数据传输失败，请稍后手动重试！" + resultMap.get("MESSAGE"));
            }
        } catch (Exception e) {
            throw new RuntimeException("数据传输失败请检查数据后重试！" + e.getMessage());
        }
    }
}
