package com.huigou.topsun.sap.suppliers.application.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import com.huigou.cache.DictUtil;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.sap.common.DefaultHttpClient;
import com.huigou.topsun.sap.common.application.SapMutualEpLogApplication;
import com.huigou.topsun.sap.suppliers.application.SuppliersDataApplication;
import com.huigou.topsun.sap.suppliers.domain.*;
import com.huigou.topsun.sap.suppliers.domain.query.SuppliersBankQueryRequest;
import com.huigou.topsun.sap.suppliers.domain.query.SuppliersBuyerQueryRequest;
import com.huigou.topsun.sap.suppliers.domain.query.SuppliersDataQueryRequest;
import com.huigou.topsun.sap.suppliers.domain.query.SuppliersPayQueryRequest;
import com.huigou.topsun.sap.suppliers.domain.vo.SuppliersBankVo;
import com.huigou.topsun.sap.suppliers.domain.vo.SuppliersBuyerVo;
import com.huigou.topsun.sap.suppliers.domain.vo.SuppliersInfoVo;
import com.huigou.topsun.sap.suppliers.domain.vo.SuppliersPayVo;
import com.huigou.topsun.sap.suppliers.repository.SuppliersBankRepository;
import com.huigou.topsun.sap.suppliers.repository.SuppliersBuyerRepository;
import com.huigou.topsun.sap.suppliers.repository.SuppliersInfoRepository;
import com.huigou.topsun.sap.suppliers.repository.SuppliersPayRepository;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bpm.ApprovalParameter;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.util.ClassHelper;
import com.huigou.util.DateUtil;
import com.huigou.util.StringUtil;
import lombok.SneakyThrows;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service("suppliersDataApplication")
public class SuppliersDataApplicationImpl extends FlowBroker implements SuppliersDataApplication {

    @Autowired
    SuppliersInfoRepository suppliersInfoRepository;
    @Autowired
    SuppliersPayRepository suppliersPayRepository;
    @Autowired
    SuppliersBuyerRepository suppliersBuyerRepository;
    @Autowired
    SuppliersBankRepository suppliersBankRepository;
    @Autowired
    DefaultHttpClient defaultHttpClient;
    @Autowired
    SapMutualEpLogApplication sapMutualEpLogApplication;

    @Override
    public Map<String, Object> slicedSuppliersDataQuery(SuppliersDataQueryRequest query) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "slicedSuppliersInfoQuery");
        QueryModel model = this.sqlExecutorDao.getQueryModel(queryDescriptor, query);
        if (StringUtil.isNotBlank(query.getBillCode())) {
            model.addCriteria(" and t.bill_code like:billCode");
            model.putLikeParam("billCode", query.getBillCode());
        }
        if (StringUtil.isNotBlank(query.getBillCode())) {
            model.addCriteria(" and t.name_Org1 like:nameOrg1");
            model.putLikeParam("nameOrg1", query.getNameOrg1());
        }
        if (StringUtil.isNotBlank(query.getPartner())) {
            model.addCriteria(" and t.partner like:partner");
            model.putLikeParam("partner", query.getPartner());
        }
        if (query.getStatus() != null) {
            model.addCriteria(" and t.status =:status");
            model.putParam("status", query.getStatus());
        }
        if (StringUtil.isNotBlank(query.getApplyType())) {
            model.addCriteria(" and t.apply_type =:applyType");
            model.putParam("applyType", query.getApplyType());
        }
//        model.putDictionary("maintenanceType", DictUtil.getDictionary("maintainType"));
//        model.putDictionary("baseUnit", DictUtil.getDictionary("measuringUnit"));
//        model.putDictionary("itemAttributes", DictUtil.getDictionary("suppliesType"));
//        model.putDictionary("businessUsage", DictUtil.getDictionary("serviceusage"));
        model.putDictionary("status", DictUtil.getDictionary("bizBillStatus"));
        model.addCriteria(" order by bill_code desc ");
        return this.sqlExecutorDao.executeSlicedQuery(model);
    }

    @SneakyThrows
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();

        SuppliersInfo suppliersInfo = getBizEntity(SuppliersInfo.class);
        List<SuppliersPay> suppliersPayList = getBizEntities(SuppliersPay.class, "suppliersPayList");
        List<SuppliersBuyer> suppliersBuyerList = getBizEntities(SuppliersBuyer.class, "suppliersBuyerList");
        List<SuppliersBank> suppliersBankList = getBizEntities(SuppliersBank.class, "suppliersBankList");

        if (suppliersInfo.isNew()) {
            suppliersInfo.setStatusId(BizBillStatus.APPLYING.getId());
        } else {
            suppliersInfo = (SuppliersInfo) commonDomainService.loadAndFillinProperties(suppliersInfo);
        }
        suppliersInfo = suppliersInfoRepository.save(suppliersInfo);

        for (SuppliersPay pay : suppliersPayList) {
            if (StringUtil.isBlank(pay.getSId())) {
                pay.setSId(suppliersInfo.getId());
                this.suppliersPayRepository.save(pay);
            } else if (!(suppliersInfo.getId().equals(pay.getSId()))) { //从新增带过来的
                SuppliersPay modifyPay = new SuppliersPay();
                ClassHelper.copyProperties(pay, modifyPay);
                modifyPay.setId(UUID.randomUUID().toString());
                modifyPay.setSId(suppliersInfo.getId());
                this.suppliersPayRepository.save(modifyPay);
            } else {
                pay.setSId(suppliersInfo.getId());
                this.suppliersPayRepository.save(pay);
            }

        }
        for (SuppliersBuyer buyer : suppliersBuyerList) {

            if (StringUtil.isBlank(buyer.getSId())) {
                buyer.setSId(suppliersInfo.getId());
                this.suppliersBuyerRepository.save(buyer);
            } else if (!(suppliersInfo.getId().equals(buyer.getSId()))) { //从新增带过来的
                SuppliersBuyer modifyBuyer = new SuppliersBuyer();
                ClassHelper.copyProperties(buyer, modifyBuyer);
                modifyBuyer.setId(UUID.randomUUID().toString());
                modifyBuyer.setSId(suppliersInfo.getId());
                this.suppliersBuyerRepository.save(modifyBuyer);
            } else {
                buyer.setSId(suppliersInfo.getId());
                this.suppliersBuyerRepository.save(buyer);
            }
        }

        for (SuppliersBank bank : suppliersBankList) {

            if (StringUtil.isBlank(bank.getSId())) {
                bank.setSId(suppliersInfo.getId());
                this.suppliersBankRepository.save(bank);
            } else if (!(suppliersInfo.getId().equals(bank.getSId()))) { //从新增带过来的
                SuppliersBank modifyBank = new SuppliersBank();
                ClassHelper.copyProperties(bank, modifyBank);
                modifyBank.setId(UUID.randomUUID().toString());
                modifyBank.setSId(suppliersInfo.getId());
                this.suppliersBankRepository.save(modifyBank);
            } else {
                bank.setSId(suppliersInfo.getId());
                this.suppliersBankRepository.save(bank);
            }
        }

        return suppliersInfo.getId();
    }

    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        // 返回业务数据给流程实例，
        return ClassHelper.toMap(suppliersInfoRepository.getOne(bizId));
    }

    @Override
    protected void onBeforeComplete(DelegateTask delegateTask) {
        super.onBeforeComplete(delegateTask);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        if (this.isApplyProcUnit(delegateTask) && this.getApprovalParameter().isAdvanceProcessAction()) {
            SuppliersInfo suppliersInfo = suppliersInfoRepository.findOne(bizId);
            suppliersInfo.setStatusId(BizBillStatus.APPROVING.getId());
            suppliersInfoRepository.save(suppliersInfo);
        }
    }

    //流程已办
    @Override
    public void onAfterComplete(DelegateTask delegateTask) {
        super.onAfterComplete(delegateTask);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        //  SuppliersInfo suppliersInfo = suppliersInfoRepository.findOne(bizId);

    }

    @Override
    public void onComplete(DelegateTask delegateTask) {
        super.onComplete(delegateTask);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        SuppliersInfo suppliersInfo = suppliersInfoRepository.findOne(bizId);
        //审批不通过
        ApprovalParameter parameter = getApprovalParameter();
        // if (ObjectUtils.equals(parameter.getHandleResult(), HandleResult.DISAGREE.id)) {   //审批不通过 或者退回
        //  }
        if (this.isApplyProcUnit(delegateTask)) {   //申请后直接提交
        }
    }

    /**
     * 流程实例结束
     *
     * @param delegateExecution
     */
    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        BizBillStatus status = approvePassed() ? BizBillStatus.COMPLETED : BizBillStatus.ABORTED;
        SuppliersInfo suppliersInfo = suppliersInfoRepository.findOne(bizId);
        suppliersInfo.setStatusId(status.getId());
        suppliersInfoRepository.save(suppliersInfo);

        List<SuppliersPay> payList = this.suppliersPayRepository.findAllBySId(suppliersInfo.getId());

        List<SuppliersBuyer> buyerList = this.suppliersBuyerRepository.findAllBySId(suppliersInfo.getId());

        List<SuppliersBank> bankList = this.suppliersBankRepository.findAllBySId(suppliersInfo.getId());

        sendSapData(suppliersInfo, payList, buyerList, bankList);

    }

    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        SuppliersInfo suppliersInfo = suppliersInfoRepository.findOne(bizId);
        suppliersInfo.setStatusId(BizBillStatus.ABORTED.getId());
        suppliersInfoRepository.save(suppliersInfo);
    }

    /**
     * @param delegateTask
     * @param destActivityId 回退到的目标节点id
     */
    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        super.onBack(delegateTask, destActivityId);
        if ("Apply".equalsIgnoreCase(destActivityId)) {
            String bizId = delegateTask.getExecution().getProcessBusinessKey();
            SuppliersInfo suppliersInfo = suppliersInfoRepository.findOne(bizId);
            suppliersInfo.setStatusId(BizBillStatus.APPLYING.getId());
            suppliersInfoRepository.save(suppliersInfo);
        }
    }


    /**
     * 远程调用sap接口，传递数据
     */
    public void sendSapData(SuppliersInfo suppliersInfo, List<SuppliersPay> payList, List<SuppliersBuyer> buyerList, List<SuppliersBank> bankList) {
        SuppliersInfoVo suppliersInfoVo = new SuppliersInfoVo();
        BeanUtil.copyProperties(suppliersInfo, suppliersInfoVo);
        suppliersInfoVo.setKtokk(suppliersInfoVo.getRltgr());
        suppliersInfoVo.setUpdkz("I");
        if (suppliersInfo.getApplyType().equals("1")) { //更新
            suppliersInfoVo.setUpdkz("U");
        }
        if (suppliersInfo.getApplyType().equals("-1")) { //删除
            suppliersInfoVo.setUpdkz("D");
        }
        List<SuppliersPayVo> payVoList = new ArrayList<>();
        for (SuppliersPay suppliersPay : payList) {
            SuppliersPayVo payVo = new SuppliersPayVo();
            ClassHelper.copyProperties(suppliersPay, payVo);
            payVoList.add(payVo);
        }
        suppliersInfoVo.setSuppliersPayList(payVoList);
        List<SuppliersBuyerVo> buyerVoList = new ArrayList<>();
        for (SuppliersBuyer suppliersBuyer : buyerList) {
            SuppliersBuyerVo buyerVo = new SuppliersBuyerVo();
            ClassHelper.copyProperties(suppliersBuyer, buyerVo);
            buyerVoList.add(buyerVo);
        }
        suppliersInfoVo.setSuppliersBuyerList(buyerVoList);
        List<SuppliersBankVo> bankVoList = new ArrayList<>();
        for (SuppliersBank suppliersBank : bankList) {
            SuppliersBankVo bankVo = new SuppliersBankVo();
            ClassHelper.copyProperties(suppliersBank, bankVo);
            bankVoList.add(bankVo);
        }
        suppliersInfoVo.setSuppliersBankList(bankVoList);
        ObjectMapper objectMapper = new ObjectMapper();
        String url = "CUD_SUPPLY/SUPPLY_DATA";
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put("businessType", "suppliersInfo");
        resultMap.put("businessId", suppliersInfo.getId());
        resultMap.put("parameter", JSON.toJSONString(suppliersInfoVo));
        List<SuppliersInfoVo> suppliersInfoVos = new ArrayList<>();
        suppliersInfoVos.add(suppliersInfoVo);
        try {
            String result = defaultHttpClient.execute(suppliersInfoVos, url);
            List<Map<String, Object>> resultList = objectMapper.readValue(result, new TypeReference<List<Map<String, Object>>>() {
            });
            resultMap = resultList.get(0);
            resultMap.put("sequence", resultMap.get("row"));
            sapMutualEpLogApplication.saveSapMutualEpLog(resultMap);
            if ("S".equals(resultMap.get("TYPE"))) {
                suppliersInfo.setPartner((String) resultMap.get("MESSAGE_V1"));
                suppliersInfoRepository.save(suppliersInfo);
            } else {
                throw new RuntimeException("数据传输失败，请稍后手动重试！" + resultMap.get("MESSAGE"));
            }
        } catch (Exception e) {
            resultMap.put("type", "E");
            resultMap.put("message", e.getMessage());
            sapMutualEpLogApplication.saveSapMutualEpLog(resultMap);
            throw new RuntimeException("数据传输失败请检查数据后重试！" + e.getMessage());
        }


    }

    /**
     * 设置任务名称
     */
    @Override
    protected void setTaskDescription(DelegateTask delegateTask) {
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        delegateTask.setDescription(this.getApprovalSubjectName(bizId));
    }

    /**
     * 获取任务标题
     *
     * @param bizId
     * @return
     */
    private String getApprovalSubjectName(String bizId) {
        SuppliersInfo suppliersInfo = this.suppliersInfoRepository.findOne(bizId);
        //查询 单据日期
        String fillinDateStr = DateUtil.getDateFormat("yyyy-MM-dd HH:mm:ss", suppliersInfo.getFillinDate());
        //设置标题
        // String wantGoodsNameText = DictUtil.getDictionaryDetailText("A0051", wantGoods.getCustomerName());
        if (suppliersInfo.getApplyType().equals("1")) {
            return String.format("%s-%s(%s)", "供应商主数据修改", "", fillinDateStr);
        } else if (suppliersInfo.getApplyType().equals("-1")) {
            return String.format("%s-%s(%s)", "供应商主数据删除", "", fillinDateStr);
        } else {
            return String.format("%s-%s(%s)", "供应商主数据申请", "", fillinDateStr);
        }
    }

    @Override
    public SuppliersInfo loadSuppliersDataById(String id) {

        SuppliersInfo suppliersInfo = suppliersInfoRepository.findOne(id);
        return suppliersInfo;
    }

    @Override
    public Map<String, Object> loadSuppliersPayDatas(SuppliersPayQueryRequest queryRequest) {

        //SuppliersInfo suppliersInfo=suppliersInfoRepository.findOne(queryRequest.getSId());
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "slicedSuppliersPayList");
        // SuppliersPayQueryRequest queryRequest=new  SuppliersPayQueryRequest();
        // queryRequest.setSId(sId);
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        // queryModel.putDictionary("konwa", DictUtil.getDictionary("currency"));
        queryModel.addCriteria(" and t.s_id =:sId");
        queryModel.putParam("sId", queryRequest.getSId());
        return this.sqlExecutorDao.executeSlicedQuery(queryModel);
    }

    @Override
    public Map<String, Object> loadSuppliersBuyerDatas(SuppliersBuyerQueryRequest queryRequest) {
        //return  this.suppliersBuyerRepository.findBysId(sId);
        // Sort sort = new Sort(Sort.Direction.ASC, "sequence");
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "slicedSuppliersBuyerList");

        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        queryModel.addCriteria(" and t.s_id =:sId");
        queryModel.putParam("sId", queryRequest.getSId());
        return this.sqlExecutorDao.executeSlicedQuery(queryModel);
    }

    @Override
    public Map<String, Object> loadSuppliersBankDatas(SuppliersBankQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "slicedSuppliersBankList");
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        queryModel.addCriteria(" and t.s_id =:sId");
        queryModel.putParam("sId", queryRequest.getSId());
        return this.sqlExecutorDao.executeSlicedQuery(queryModel);
    }

    //删除数据
    @Override
    public void deleteSuppliersDataByIds(List<String> ids) {
        for (String id : ids) {
            List<SuppliersPay> suppliersPayList = this.suppliersPayRepository.findAllBySId(id);
            for (SuppliersPay suppliersPay : suppliersPayList) {
                suppliersPayRepository.delete(suppliersPay);
            }
            List<SuppliersBuyer> suppliersBuyerList = this.suppliersBuyerRepository.findAllBySId(id);
            for (SuppliersBuyer suppliersBuyer : suppliersBuyerList) {
                suppliersBuyerRepository.delete(suppliersBuyer);
            }
            List<SuppliersBank> suppliersBankList = this.suppliersBankRepository.findAllBySId(id);
            for (SuppliersBank suppliersBank : suppliersBankList) {
                suppliersBankRepository.delete(suppliersBank);
            }
            this.suppliersInfoRepository.delete(id);
        }
    }

    @Override
    public List<SuppliersInfo> findDeletedSuppliersDataById(String id) {

        List<SuppliersInfo> list = suppliersInfoRepository.findDeletedById(id);
        return list;
    }
}
