package com.huigou.topsun.sap.workcenter.application.impl;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import com.huigou.topsun.sap.common.HttpClient;
import com.huigou.topsun.sap.workcenter.repository.SapWorkCenterRepository;
import com.huigou.topsun.sap.workcenter.application.SapWorkCenterApplication;
import com.huigou.topsun.sap.workcenter.domain.SapWorkCenter;
import com.huigou.topsun.util.PersistTool;
import com.huigou.topsun.util.TestClientHttp;
import com.huigou.util.JSONUtil;
import com.huigou.util.SDO;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.*;

/**
 * @author YourUserName
 */
@Service
@RequiredArgsConstructor
public class SapWorkCenterApplicationImpl implements SapWorkCenterApplication {

    private final ObjectMapper objectMapper;

    @PersistenceContext(unitName = "system")
    private EntityManager entityManager;
    private final PersistTool persist;

    private final SapWorkCenterRepository rep;
    private final HttpClient httpClient;
    private final SapWorkCenterRepository sapWorkCenterRepository;

    /***
     * @description 发送请求到 SAP 系统，获取工作中心数据
     * @return java.util.ArrayList<java.util.Map < java.lang.String, java.lang.Object>>
     * @author qinzhenguan
     * @createDate 2024/1/17 15:44
     */
    @Override
    public ArrayList<Map<String, Object>> findWorkCenter() {
        String execute;
        HashMap<String, Object> param = new HashMap<>(12);
        List<SapWorkCenter> workCenters;
        try {
            execute = TestClientHttp.execute(param, "http://192.168.3.91:8081/topsun-xt/sapWorkCenter/queryWorkCenter.ajax");
            String resultStr = JSONObject.parseObject(execute).get("data").toString();
            workCenters = objectMapper.readValue(resultStr, new TypeReference<List<SapWorkCenter>>() {});
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (workCenters.size() == 0) {
            return null;
        }
        return objectMapper.convertValue(workCenters, new TypeReference<ArrayList<Map<String, Object>>>() {
        });
    }

    /***
     * @description  发送请求到 SAP 系统，保存工作中心数据
     * @param sdo   提交的数据集
     * @return java.util.Map<java.lang.String, java.lang.Object>
     * @author qinzhenguan
     * @createDate 2024/1/18 11:20
     */
    @Override
    public String saveWorkCenterDetail(SDO sdo) {
        String execute;
        HashMap<String, Object> param = new HashMap<>(12);
        SapWorkCenter sapWorkCenter = sdo.toObject(SapWorkCenter.class);
        param.put("data", sdo.getProperties());
        try {

            execute = httpClient.execute(param,"http://192.168.3.109:8000/sap/bc/erp_ep/cud_arbpl/arbpl_data?sap-client=300");
            List<Map> maps = JSONObject.parseArray(execute, Map.class);
            String message = (String)maps.get(0).get("MESSAGE");
            String type = (String)maps.get(0).get("TYPE");
            sapWorkCenter.setType(type);
            sapWorkCenter.setMessage(message);
            sapWorkCenterRepository.save(sapWorkCenter);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return sapWorkCenter.getId();
    }

    @Override
    @Transactional(transactionManager = "transactionManager", rollbackFor = { Exception.class })
    public boolean saveWorkCenter(SapWorkCenter workCenter) {
        Set<Object> set = new HashSet<>();
        set.add(workCenter);
        persist.persistEntitys(set);
        entityManager.flush();
        entityManager.clear();
//        String str = null;
//        str.length();
        System.out.println("GG");
        return false;
    }

    /**
     * description 模拟 SAP 接口 (临时)
     * @return java.lang.String
     * @author qinzhenguan
     * @createDate 2024/1/17 14:57
     */
    @Override
    public String queryWorkCenter() throws JsonProcessingException {
        List<SapWorkCenter> list = rep.findAll();
        return list.size() > 0 ? objectMapper.writeValueAsString(list) : "";
    }



}
