package com.huigou.topsun.util;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.huigou.topsun.config.CloseableClientHttpRequest;
import org.springframework.http.HttpMethod;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.http.client.SimpleClientHttpRequestFactory;

import java.io.*;
import java.net.URI;
import java.nio.charset.StandardCharsets;

/**
 * TestClientHttp 用于测试的 ClientHttp
 *
 * @author qinzhenguan
 * @createDate 2024/01/19 15:43
 **/
public class TestClientHttp {

    /***
     * description  模拟发送请求到 SAP
     * @param param 请求体 body
     * @param url  请求地址
     * @return java.lang.String
     * @author qinzhenguan
     * @createDate 2024/1/17 17:01
     */
    public static String execute(Object param, String url) throws IOException {
        String str = "";
        try {
            SimpleClientHttpRequestFactory requestFactory = new SimpleClientHttpRequestFactory();
            requestFactory.setConnectTimeout(5000);
            requestFactory.setReadTimeout(50000);

            URI uri = URI.create(url);
            HttpMethod method = HttpMethod.POST;

            // ------------------------  header  ------------------------
            try (CloseableClientHttpRequest request = new CloseableClientHttpRequest(requestFactory.createRequest(uri, method))) {
                request.addHeader("Content-type", "application/json;charset=utf-8");

                // ------------------------  body  ------------------------
                //JSONObject body = new JSONObject();
                if(param != null) {
                    String bodyText = new ObjectMapper().writeValueAsString(param);
                    try (OutputStream requestBody = request.getBody()) {
                        requestBody.write(bodyText.getBytes());
                    }
                }

                // ------------------------  response  ------------------------
                try (ClientHttpResponse response = request.execute();
                     InputStream ist = response.getBody();
                     InputStreamReader istr = new InputStreamReader(ist, StandardCharsets.UTF_8);
                     BufferedReader bfreader = new BufferedReader(istr)) {

                    str = bfreader.readLine();
                    System.out.println(str);
                }
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return str;
    }

}